_G.filters = setmetatable({}, {
    __index = function(t, k)
        local filter = require('filter/'..k)
        filter.name = k
        return filter:new()
    end,
    __newindex = function()
        error('read only')
    end,
    __len = FILTER_COUNT,
})

eqe.getbands = function()
    local ans = {}

    table.insert(ans, {k = 'preamp', v = eqe.preamp, i = 0})

    for i=1,FILTER_COUNT() do
        local band = FILTER_GET(i)

        table.insert(ans, {k = 'Q', v = band.Q, i = i})
        table.insert(ans, {k = 'frequency', v = band.frequency, i = i})
        if band.class.gain then
            table.insert(ans, {k = 'gain', v = band.gain, i = i})
        end
    end

    return ans
end

-- this is pretty hacky.
-- todo: this should preserve the recursive buffers.
-- super todo: this should sort automatically while you're changing the slider.
eqe.sort = function()
    local ans = {}
    for i=1,#eqe do
        local t = {}
        for _,k in ipairs{
            'name',
            'frequency',
            'Q',
            'gain',
            'channel',
        } do
            t[k] = eqe[i][k]
        end
        table.insert(ans, t)
    end
    table.sort(ans, function(a, b)
        return a.frequency < b.frequency
    end)
    ans.preamp = eqe.preamp
    eqe.save(nil, ans)
    eqe.load(nil)
end

eqe.raw = FILTER_RUN_RAW
eqe.insert = FILTER_INSERT
function eqe.update(...)
    return FILTER_UPDATE(...)
end
function eqe.flatten(...)
    return FILTER_FLATTEN(...)
end

local get = {}
get.preamp = FILTER_GET_PREAMP
get.sample_rate = FILTER_GET_SAMPLE_RATE

local set = {}
set.preamp = FILTER_SET_PREAMP

eqe.attr = {}

setmetatable(eqe, {
    __index = function(t, k)
        local f = get[k]
        if f then
            return f()
        else
            return FILTER_GET(k)
        end
    end,
    __newindex = function(t, k, v)
        local f = set[k]
        if f then
            f(v)
        elseif type(k) == 'number' then
            if type(v) == 'table' then
                FILTER_INSERT(k, v)
            elseif v == nil then
                FILTER_REMOVE(k)
            else
                error('invalid type')
            end
        elseif get[k] then
            error('readonly')
        else
            rawset(t, k, v)
        end
    end,
    __len = FILTER_COUNT,
})

return eqe
