PRESET_FOLDER = '/var/tweak/com.r333d.eqe/db/presets'
function preset_file(name)
    name = name or 'CURRENT_BANDS'
    return PRESET_FOLDER..'/'..name..'.lua'
end

local esc = require 'str_esc'

function eqe.save(name, input)
    input = input or eqe
    name = name and string.gsub(name, '%/', '\\')

    local path = preset_file(name)
    local buf = {}
    local function write(s)
        buf[#buf + 1] = s
    end
    write('return {')
    write('    {')
    write('        name = "preamp",')
    write('        gain = '..input.preamp..',')
    write('    },')
    for i=1,#input do
        local filter = input[i]
        write('    {')
        write('        name = "'..filter.name..'",')
        write('        frequency = '..filter.frequency..',')
        write('        Q = '..filter.Q..',')
        if filter.gain then
            write('        gain = '..filter.gain..',')
        end
        if filter.channel then
            write('        channel = '..filter.channel..',')
        end
        write('    },')
    end
    write('}')
    if not name and input == eqe and next(eqe.attr) then
        write(', ')
        write(esc(eqe.attr))
    end
    IPCD('WRITE_FILE('..esc(path)..','..esc(table.concat(buf))..')')
    IPCD('UPDATE_PRESET('..esc(name or 'CURRENT_BANDS')..','..esc(path)..')')
end

function eqe.load(name, target)
    return eqe.loadraw(preset_file(name), target)
end

function eqe.loadraw(path, target)
    target = target or eqe
    local f = assert(loadfile(path), path)

    local t, attr
    do
        local _ENV = {} -- prevent "heres my preset" OSHIT PWNED situations
        t, attr = f()
    end
    -- clear all bands
    if target == eqe then
        local to_insert = {}
        for k,info in pairs(t) do
            if info.name == 'preamp' then
                eqe.preamp = info.gain
            else
                local filter = filters[info.name]
                for k,v in pairs(info) do
                    filter[k] = v
                end
                table.insert(to_insert, filter)
            end
        end
        if attr then
            for k,v in pairs(attr) do
                eqe.attr[k] = v
            end
        end
        eqe.insert(nil, to_insert, true)
        eqe.update()
        if name then
            eqe.save()
        end
    else
        local n = #target
        for i=1,n do
            target[#target] = nil
        end
        target.preamp = 0
        for _,info in pairs(t) do
            if info.name == 'preamp' then
                target.preamp = info.gain
            else
                local filter = filters[info.name]
                for k,v in pairs(info) do
                    filter[k] = v
                end
                target[#target + 1] = filter
            end
        end
    end
    return target
end
